#include <stdio.h>
#include <htc.h>
#include "lcd.h"
#include <string.h>
#include <stdlib.h>
#include "sci.h"
#include <math.h>
#define _XTAL_FREQ 20000000

__CONFIG( HS & WDTDIS & PWRTEN & BOREN & LVPDIS & WRTEN & DEBUGDIS & UNPROTECT );


//Global variables... shouldn't really have these but it works!
char rxstring[80];
char * pos;	//Pointer for string position
char FEC[3];
char SR[5];
char VPID[5];
char APID[5];
char CALLSIGN[11];
char FREQ[16];  //16bit int to string with itoa
char MODE;




void delayS(char seconds){		// Delay function to get roughly seconds
	for (char a=0;a<seconds;a++){
		for (char n=0;n<100;n++){
		__delay_ms(10);
		}
	}
}

void delaymS(char msec){		// Delay function to get roughly milliseconds
		for (char n=0;n<msec;n++){
		__delay_ms(1);
		}
}




//SPI routine for Ultram VCO
void spi_write(long int reg){
	//Data = RA0
	//Clock = RA1
	//Load = RA2


	long int tmp;		//24bits of 32bit int used for VCO registers
	char n;
	for(n=24;n>0;n--){
		tmp = 1;
		tmp = tmp << (n-1);		//create mast for bit selection
		tmp = reg & tmp;		//AND mask with register
		tmp = tmp >> n-1;  		//Shift selected bit to LSB
		RA0 = tmp;				//output bit to DATA port
		delaymS(1);

		//clock data bit
		RA1=1;
		delaymS(1);
		RA1=0;
	}

	//Load register
	RA2=1;
	delaymS(1);
	RA2=0;

}


void putfreq(void){
		lcd_putch(FREQ[0]);
		lcd_putch(FREQ[1]);
		lcd_putch(FREQ[2]);
		lcd_putch(FREQ[3]);
		lcd_putch('.');
		lcd_putch(FREQ[4]);


}


void srcopts(void){		//Source display modes function to save space!
	switch(MODE){
		case 0:
			lcd_puts("Live");
			break;
		case 1:
			lcd_puts("SDcard");
			break;
		case 2:
			lcd_puts("Carrier   ");
			break;
		case 3:
			lcd_puts("LSB       ");
			break;
		case 4:
			lcd_puts("USB       ");
			break;
		case 5:
			lcd_puts("In Phase  ");
			break;
		case 6:
			lcd_puts("ALT1      ");
			break;
		case 7:
			lcd_puts("ALT2      ");
			break;
		case 8:
			lcd_puts("ALT4      ");
			break;
	}


}



// Request settings from Digilite
char getDLsettings(void){

	sci_PutByte('A');
	sci_PutByte(':');
	sci_PutByte('?');
	sci_PutByte(13);

	char n = 0;
	while(1){
		if(FERR){					//if frame error get errored byte to clear error
			sci_GetByte();
		}else{
			rxstring[n] = sci_GetByte();		//grab next byte
			if (rxstring[n] == ']'){break;}		//jump out of while if end if line char received (']')
			if (rxstring[n] == 1){break;}		//break on timeout return from get byte  //buggy!
			if(n>78){break;}					// to stop over-flow of buffer!
			sci_CheckOERR();					//check for overrun error, if exists will clear.
			n++;		
		}
	}
//	pos = strstr(rxstring,"[A:100 Version");		//check received string is valid
	if(strstr(rxstring,"[A:100 Version")){
		pos = strstr(rxstring,"SR = ");	//Wait for string within rx data	
		for (n=0;n<4;n++){	//extract required chars from string
			SR[n]=*(pos+n+5);
		}
		pos = strstr(rxstring,"FEC = ");	//Wait for string within rx data		
		FEC[0]=*(pos+6);
		FEC[1]=*(pos+8);	//skip '/'
		return 0;		//found, continue
	}else{
		return 1;		//not found, retry
	}
}


//changes digilite settings
void setDL(void){

	sci_PutByte('A');	//send update request string to DigiLite containing new SR and FEC
	sci_PutByte(':');
	sci_PutByte('S');
	sci_PutByte('R');
	sci_PutByte('F');
	for(char n=0;n<4;n++){
		sci_PutByte(SR[n]);
	}
	sci_PutByte(',');
	if(MODE>1){			//IF test modes selected
		sci_PutByte('-');
		sci_PutByte(MODE-1+48);
		sci_PutByte(13);	
	}else{
		sci_PutByte(FEC[0]);
		sci_PutByte(13);
		delaymS(250);

		while(getDLsettings())	//check new settings have been accepted, repeats as sometimes timing issue*****
			continue;
		
	}

}



void get808string(void){
	char n = 0;

	while(1){
		rxstring[n] = getch();	//grab next byte from SOFTWARE serial port
		if (rxstring[n] == '.'){break;}	//jump out of while if  end if line char received ('.')
		if (rxstring[n] == 1){break;}
		if(n>78){break;}
		n++;
	}
}





void send808string(void){		//send SR,FEC,Video PID,Audio PID,Callsign to MK808

	if (MODE ==0){
		char n;
		putch('#');							
		putch('#');
		for(n=0;n<4;n++){putch(SR[n]);}
		putch(',');
		putch(FEC[0]);
		putch('/');
		putch(FEC[1]);
		putch(',');
		for(n=0;n<4;n++){putch(VPID[n]);}
		putch(',');	
		for( n=0;n<4;n++){putch(APID[n]);}
		putch(',');
		for(n=0;n<10;n++){putch(CALLSIGN[n]);}
		putch(10);	//newline char
	}else{
		putch('s');
		putch('t');
		putch('o');
		putch('p');
		putch(10);
	}

}


//Set VCO Frequency	
void setfreq (char * FREQ){

	
	int F;
	F = atoi(FREQ);		//convert FREQ string to int to allow calculation of registers
	
	long int B;  		//long needed for left shift below
	char A;
	B = F/32;			//B from frequecny 
	A = F - (32*B);		//A from frequency and B

	long int N;
	N = A<<2 | (B<<8) | 2;  //Creates N register from A and B.

	spi_write(3146129); 	//R reg  1100000000000110010001  //100KHz step
	delaymS(5);
	spi_write(9431332); 	//Control reg 
	delaymS(5);
	spi_write(N); 			//N reg    

}


//Config Digilite, MK808 and VCO
void configure(void){
	lcd_clear();
	lcd_gotorow(1);
	lcd_puts("Configuring");
	lcd_gotorow(2);
	lcd_puts("DigiLite");

	delayS(1);


//Setup DigiLite with memory settings.
	if(MODE==0){
	setDL();
	}

	if(MODE>1){
	setDL();
	}



	lcd_clear();
	lcd_gotorow(1);
	lcd_puts("Configuring");
	lcd_gotorow(2);
	lcd_puts("MK808");

	delayS(1);


//send MK808 settings... 
	char n = 0;
	while(!n){			
		send808string();
		get808string();	//get string from MK808 when it's ready...
		if(MODE==0){
			pos = strstr(rxstring,"Config ok");
			if(pos){n=1;}

		}else{
			pos = strstr(rxstring,"Stopped");	//check it is valid
			if(pos){n=1;}

		}
	}


//Set VCO frequency
	setfreq(FREQ);



//Main display output
	lcd_clear();
	lcd_gotorow(1);	// select first line
	lcd_putch(CALLSIGN[0]);
	lcd_putch(CALLSIGN[1]);
	lcd_putch(CALLSIGN[2]);
	lcd_putch(CALLSIGN[3]);
	lcd_putch(CALLSIGN[4]);
	lcd_putch(CALLSIGN[5]);
	lcd_putch(CALLSIGN[6]);
	lcd_putch(CALLSIGN[7]);
	lcd_putch(CALLSIGN[8]);
	lcd_putch(CALLSIGN[9]);

	switch(MODE){
		case 0:
			lcd_puts("Live");
			break;
		case 1:
			lcd_puts("SDcard");
			break;
		default:
			lcd_puts("TstMod");
			break;
	}





	lcd_gotorow(2);	// select first line

	if(MODE==0){
		lcd_puts(SR);
		lcd_putch(' ');
		lcd_putch(FEC[0]);
		lcd_putch('/');
		lcd_putch(FEC[1]);
		lcd_putch(' ');
		lcd_putch(' ');
		putfreq();	//function to save space!
	}
	if(MODE==1){
		lcd_puts("SD SR/FEC ");
		putfreq();
	}
	
	if(MODE>1){

		srcopts();//function to save space!
		putfreq();//function to save space!
	}

//Set SDcard source if required
	if(MODE==1){
		RA3 = 0;
	}else{
		RA3=1;
	}



}





void menu (void){

	int SRopt[8] = {1000,1333,1500,1667,2000,4000,4166};		//Digilite cannot do exactly 4167 hence 4166(.667) but it works fine.
	int FECopt[6] = {12,23,34,56,78};

	//Menu actions.........

	char cursorpos;
	char changed;
	char n;



	//SR menu option
	if(RC0==0){			//Menu button pressed
		lcd_clear();
		lcd_gotorow(1);	// select first line
		lcd_puts("Set SR");
		lcd_gotorow(2);	// select first line
		lcd_putch(SR[0]);
		lcd_putch(SR[1]);
		lcd_putch(SR[2]);
		lcd_putch(SR[3]);
		
		changed = 0;


		while(RC2){		//while Enter NOT pressed
			if(RC1==0){	//when adjust pressed
				delaymS(200);
				char tmp[16];
				n=0;
				while(n<7){
					itoa(tmp,SRopt[n],10);
					if(strcmp(tmp,SR)==0){
						if(n==6){
							itoa(tmp,SRopt[0],10);
							SR[0] = tmp[0];
							SR[1] = tmp[1];
							SR[2] = tmp[2];
							SR[3] = tmp[3];
							break;
						}else{
							itoa(tmp,SRopt[(n+1)],10);
							SR[0] = tmp[0];
							SR[1] = tmp[1];
							SR[2] = tmp[2];
							SR[3] = tmp[3];
							break;
						}
					}
					n++;
				}

				lcd_gotorow(1);	// select first line
				lcd_puts("Set SR");
				lcd_gotorow(2);	// select first line
				lcd_putch(SR[0]);
				lcd_putch(SR[1]);
				lcd_putch(SR[2]);
				lcd_putch(SR[3]);
				changed=1;
			}	
		}


	


	//FEC menu option
		lcd_clear();
		lcd_gotorow(1);	// select first line
		lcd_puts("Set FEC");
		lcd_gotorow(2);	// select first line

		
		lcd_putch(FEC[0]);
		lcd_putch('/');
		lcd_putch(FEC[1]);
		
		delaymS(200);



		while(RC2){		//while Enter NOT pressed
			if(RC1==0){	//when adjust pressed
				delaymS(200);
				char tmp[16];
				n=0;

				while(n<5){
					itoa(tmp,FECopt[n],10);

					if(strcmp(tmp,FEC)==0){

						if(n==4){

							itoa(tmp,FECopt[0],10);
							FEC[0] = tmp[0];
							FEC[1] = tmp[1];

							break;
						}else{
							itoa(tmp,FECopt[n+1],10);
							FEC[0] = tmp[0];
							FEC[1] = tmp[1];
							break;
						}
					}
					n++;
				}

				lcd_gotorow(1);	// select first line
				lcd_puts("Set FEC");
				lcd_gotorow(2);	// select first line

				lcd_putch(FEC[0]);
				lcd_putch('/');
				lcd_putch(FEC[1]);
				
				changed=1;
			}
				
		}




	//CALLSIGN menu option
		lcd_clear();
		lcd_gotorow(1);	// select first line
		lcd_puts("Set Callsign");
		lcd_gotorow(2);	// select second line	
		for(n=0;n<10;n++){
			lcd_putch(CALLSIGN[n]);
		}


		lcd_putch('<');
		delaymS(200);
		lcd_gotorow(2);
		


		cursorpos = 0;
		while(RC2){		//while Enter NOT pressed
			if(RC1==0){	//when adjust pressed
				delaymS(200);
				lcd_write(0x0E);	
				while(cursorpos < 10){
					delaymS(250);
					while(RC2){		//while Enter NOT pressed
						if(RC1==0){	//when adjust pressed
							delaymS(200);
							if(CALLSIGN[cursorpos] > 89){
									CALLSIGN[cursorpos] = 32;
							}else{
								if(CALLSIGN[cursorpos] == 32){
									CALLSIGN[cursorpos] = 47;
								}else{
									CALLSIGN[cursorpos] ++;
								}
							}
							lcd_gotorow(2);	// select second line	
							for(n=0;n<10;n++){
								lcd_putch(CALLSIGN[n]);
							}
							lcd_putch('<');
							lcd_goto(0x80 + 0x40 + cursorpos);
							changed=1;
						}
					}
					cursorpos++;
					lcd_goto(0x80 + 0x40 + cursorpos);
				}
				lcd_write(0x0C);  //turn off cursor
			}
		}


	//Video Pid menu option
		lcd_clear();
		lcd_gotorow(1);	// select first line
		lcd_puts("Set Video PID");
		lcd_gotorow(2);	// select second line	
		lcd_puts(VPID);
		delaymS(200);
		lcd_gotorow(2);



		cursorpos = 0;
		while(RC2){		//while Enter NOT pressed
			if(RC1==0){	//when adjust pressed
				delaymS(200);
				lcd_write(0x0E);
				while(cursorpos < 4){
				delaymS(250);
					while(RC2){		//while Enter NOT pressed
						if(RC1==0){	//when adjust pressed
							delaymS(200);
							if(VPID[cursorpos] > 56){ //9
									VPID[cursorpos] = 48;  //0
							}else{
								VPID[cursorpos] ++;
							}
							lcd_gotorow(2);	// select second line	
							lcd_puts(VPID);
							lcd_goto(0x80 + 0x40 + cursorpos);
							changed=1;
						}
					}
					cursorpos++;
					lcd_goto(0x80 + 0x40 + cursorpos);
				}
				lcd_write(0x0C);  //turn off cursor
			}
		}


	//Audio Pid menu option
		lcd_clear();
		lcd_gotorow(1);	// select first line
		lcd_puts("Set Audio PID");
		lcd_gotorow(2);	// select second line	
		lcd_puts(APID);
		delaymS(200);
		lcd_gotorow(2);



		cursorpos = 0;
		while(RC2){		//while Enter NOT pressed
			if(RC1==0){	//when adjust pressed
				delaymS(200);
				lcd_write(0x0E);
				while(cursorpos < 4){
				delaymS(250);
					while(RC2){		//while Enter NOT pressed
						if(RC1==0){	//when adjust pressed
							delaymS(200);
							if(APID[cursorpos] > 56){ //9
									APID[cursorpos] = 48;  //0
							}else{
								APID[cursorpos] ++;
							}
							lcd_gotorow(2);	// select second line	
							lcd_puts(APID);
							lcd_goto(0x80 + 0x40 + cursorpos);
							changed=1;
						}
					}
					cursorpos++;
					lcd_goto(0x80 + 0x40 + cursorpos);
				}
				lcd_write(0x0C);  //turn off cursor
			}
		}



	//Frequency menu option
		lcd_clear();
		lcd_gotorow(1);	// select first line
		lcd_puts("Set Frequency");
		lcd_gotorow(2);	// select second line	
		putfreq();
		lcd_puts("MHz");
		delaymS(200);
		lcd_gotorow(2);
		


		cursorpos = 1;
		lcd_goto(0x80 + 0x40 + cursorpos);


		while(RC2){		//while Enter NOT pressed
			if(RC1==0){	//when adjust pressed
				delaymS(200);
				lcd_write(0x0E);
				while(cursorpos < 6){
				delaymS(250);
					while(RC2){		//while Enter NOT pressed
						if(RC1==0){	//when adjust pressed
							delaymS(200);
							if(cursorpos < 5){
								if(FREQ[cursorpos] > 56){ //9
										FREQ[cursorpos] = 48;  //0
								}else{
									FREQ[cursorpos] ++;
								}
							}else{
								if(FREQ[cursorpos-1] > 56){ //9
									FREQ[cursorpos-1] = 48;  //0
								}else{
									FREQ[cursorpos-1] ++;
								}
		
							}
							lcd_gotorow(2);	// select second line	
							putfreq();
								lcd_goto(0x80 + 0x40 + cursorpos);
							changed=1;
						}
					}
					cursorpos++;
					if (cursorpos == 4){ cursorpos++;}	//skip '.'
					lcd_goto(0x80 + 0x40 + cursorpos);
				}
				lcd_write(0x0C);  //turn off cursor
			}
		}




	//Source menu option
		lcd_clear();
		lcd_gotorow(1);	// select first line
		lcd_puts("Set Source");
		lcd_gotorow(2);	// select second line	
		srcopts();
		delaymS(250);




		while(RC2){		//while Enter NOT pressed
			if(RC1==0){	//when adjust pressed
				delaymS(200);
				if(MODE==8){
					MODE=0;
				}else{
					MODE++;
				}
				lcd_clear();
				lcd_gotorow(1);	// select first line
				lcd_puts("Set Source");
				lcd_gotorow(2);	// select second line	
				srcopts();
				
				changed = 1;
			}
		}





		//Save settings
		if (changed ==1){
			eeprom_write(0,FEC[0]);
			eeprom_write(1,FEC[1]);
			eeprom_write(2,SR[0]);
			eeprom_write(3,SR[1]);
			eeprom_write(4,SR[2]);
			eeprom_write(5,SR[3]);
			eeprom_write(6,VPID[0]);
			eeprom_write(7,VPID[1]);
			eeprom_write(8,VPID[2]);
			eeprom_write(9,VPID[3]);
			eeprom_write(10,APID[0]);
			eeprom_write(11,APID[1]);
			eeprom_write(12,APID[2]);
			eeprom_write(13,APID[3]);
			eeprom_write(14,CALLSIGN[0]);
			eeprom_write(15,CALLSIGN[1]);
			eeprom_write(16,CALLSIGN[2]);
			eeprom_write(17,CALLSIGN[3]);
			eeprom_write(18,CALLSIGN[4]);
			eeprom_write(19,CALLSIGN[5]);
			eeprom_write(20,CALLSIGN[6]);
			eeprom_write(21,CALLSIGN[7]);
			eeprom_write(22,CALLSIGN[8]);
			eeprom_write(23,CALLSIGN[9]);
			eeprom_write(24,FREQ[0]);
			eeprom_write(25,FREQ[1]);
			eeprom_write(26,FREQ[2]);
			eeprom_write(27,FREQ[3]);
			eeprom_write(28,FREQ[4]);
			eeprom_write(29,MODE);		
		}




	//Update DigiLite and MK808 settings
	configure();


	}
}



void main(void){

//PIC initialisation of ports ADC,Comparator etc...
	TRISB=0;
	TRISC7=1;
	TRISC6=0;
	TRISC5=1;
	TRISC4=0;
	TRISA0=0;
	TRISA1=0;
	TRISA2=0;
	TRISA3=0;	
	ADCON1 = 6;	// Disable analog pins on PORTA
	CMCON=7;
	RA0=0;	//VCO Data
	RA1=0;	//VCO Clock
	RA2=0;	//VCO Load
	RA3=1;	//SDCard select

//set tinterrupts for timer0  used for software serial port
	OPTION = 0b0111;	// prescale by 256
	T0CS = 0;			// select internal clock
	T0IE = 1;			// enable timer interrupt


//Initialise comm and lcd
	sci_Init(57600,0);	//hardware UART serial port to talk to DigiLite
	lcd_init();		// intialise LCD




 //////////////////////////////////////////////////////////////////////
//init defualt settings
if(eeprom_read(0)==255){ 	//create default settings if eeporm is empty
	FEC[0] = '1';
	FEC[1] = '2';
	SR[0] = '4';
	SR[1] = '0';
	SR[2] = '0';
	SR[3] = '0';
	VPID[0]='0';
	VPID[1]='2';
	VPID[2]='5';
	VPID[3]='6';
	APID[0]='0';
	APID[1]='2';
	APID[2]='5';
	APID[3]='7';
	CALLSIGN[0]='M';
	CALLSIGN[1]='0';
	CALLSIGN[2]='D';
	CALLSIGN[3]='T';
	CALLSIGN[4]='S';
	CALLSIGN[5]=' ';
	CALLSIGN[6]=' ';
	CALLSIGN[7]=' ';
	CALLSIGN[8]=' ';
	CALLSIGN[9]=' ';
	FREQ[0]='1';
	FREQ[1]='2';
	FREQ[2]='8';
	FREQ[3]='0';
	FREQ[4]='0';
	MODE=0;
}else{
	FEC[0] = eeprom_read(0);
	FEC[1] =  eeprom_read(1);
	SR[0] =  eeprom_read(2);
	SR[1] =  eeprom_read(3);
	SR[2] =  eeprom_read(4);
	SR[3] =  eeprom_read(5);
	VPID[0]= eeprom_read(6);
	VPID[1]= eeprom_read(7);
	VPID[2]= eeprom_read(8);
	VPID[3]= eeprom_read(9);
	APID[0]= eeprom_read(10);
	APID[1]= eeprom_read(11);
	APID[2]= eeprom_read(12);
	APID[3]= eeprom_read(13);
	CALLSIGN[0]= eeprom_read(14);
	CALLSIGN[1]= eeprom_read(15);
	CALLSIGN[2]= eeprom_read(16);
	CALLSIGN[3]= eeprom_read(17);
	CALLSIGN[4]= eeprom_read(18);
	CALLSIGN[5]= eeprom_read(19);
	CALLSIGN[6]= eeprom_read(20);
	CALLSIGN[7]= eeprom_read(21);
	CALLSIGN[8]= eeprom_read(22);
	CALLSIGN[9]= eeprom_read(23);
	FREQ[0] = eeprom_read(24);
	FREQ[1] = eeprom_read(25);
	FREQ[2] = eeprom_read(26);
	FREQ[3] = eeprom_read(27);
	FREQ[4] = eeprom_read(28);
	MODE = eeprom_read(29);
}





//Display boot up text
	lcd_gotorow(1);	// select first line
	delayS(1);
	lcd_puts("DigiLite + MK808");
	lcd_gotorow(2);	// Select second line
	lcd_puts("Controller.");
	delayS(3);

//wait for MK808 to boot up...
	lcd_clear();
	lcd_gotorow(1);	// select first line
	lcd_puts("Please Wait");
	lcd_gotorow(2);	// select first line
	lcd_puts("MK808 start up..");
	delayS(1);


	lcd_clear();
	lcd_gotorow(1);	// select first line
	lcd_puts("Please Wait");
	lcd_gotorow(2);	// select first line

	for(char n=0;n<16;n++){		//pointless loading indicater...!
		lcd_puts(".");
		delayS(1);
	}	
	

	char n=0;
	while (!n){		//Wait for response from MK808 after sending 'start' command...
		putch('s');
		putch('t');
		putch('a');
		putch('r');
		putch('t');
		putch(13);
		putch(10);
		get808string();						//get string from MK808 when it's ready...
		pos = strstr(rxstring,"started");	//check it is valid
		if(pos){n=1;}
	}

	//setup DigiLite and MK808
	configure();

	for(;;){
		menu();

	}
	
}




